import {LOCALE_ID, NgModule} from '@angular/core';

// Include the components so the types are available
import '@anjuna/core';

// Common type stuff
import {DraggableDirective} from './common/drag-drop/draggable.directive';
import {DroppableDirective} from './common/drag-drop/droppable.directive';
import {ResizerDirective} from './common/listeners/resizer.directive';
import {TransitionDirective} from './common/transition/transition.directive';
import {ZuoraDatePipe} from './common/pipes/date.pipe';

// Forms
import {BooleanValueDirective} from './forms/directives/boolean-value.directive';
import {InputValueDirective} from './forms/directives/input-value.directive';
import {DateValidatorDirective} from './forms/directives/date.directive';
import {MaxValidatorDirective} from './forms/directives/max.directive';
import {MinValidatorDirective} from './forms/directives/min.directive';
import {RequiredCheckDirective} from './forms/directives/required-check.directive';
import {SelectValueDirective} from './forms/directives/select-value.directive';
import {DatePickerDirective} from './forms/directives/date-picker.directive';
import {DateAdapter, NativeDateAdapter} from './forms/services/date-picker.service';
import {TimePickerDirective} from './forms/directives/time-picker.directive';

// Tables
import {CellDefDirective} from './table/directives/cell-def.directive';
import {ColumnDefDirective} from './table/directives/column-def.directive';
import {HeaderCellDefDirective} from './table/directives/header-cell-def.directive';
import {RowDefDirective} from './table/directives/row-def.directive';
import {TableDirective} from './table/directives/table.directive';

@NgModule({
    imports: [],
    declarations: [
        // Common
        DraggableDirective,
        DroppableDirective,
        ResizerDirective,
        TransitionDirective,
        ZuoraDatePipe,
        // Forms
        BooleanValueDirective,
        InputValueDirective,
        DatePickerDirective,
        DateValidatorDirective,
        MaxValidatorDirective,
        MinValidatorDirective,
        RequiredCheckDirective,
        SelectValueDirective,
        TimePickerDirective,
        // Tables
        CellDefDirective,
        ColumnDefDirective,
        HeaderCellDefDirective,
        RowDefDirective,
        TableDirective
    ],
    exports: [
        // Common
        DraggableDirective,
        DroppableDirective,
        ResizerDirective,
        TransitionDirective,
        ZuoraDatePipe,
        // Forms
        BooleanValueDirective,
        InputValueDirective,
        DatePickerDirective,
        DateValidatorDirective,
        MaxValidatorDirective,
        MinValidatorDirective,
        RequiredCheckDirective,
        SelectValueDirective,
        TimePickerDirective,
        // Tables
        CellDefDirective,
        ColumnDefDirective,
        HeaderCellDefDirective,
        TableDirective
    ],
    providers: [
        {
            provide: DateAdapter,
            useClass: NativeDateAdapter,
            deps: [LOCALE_ID]
        },
        ZuoraDatePipe
    ]
})
export class AnjunaCoreModule {}
