import {Directive, Input, HostBinding, HostListener} from '@angular/core';

/**
 * @description A wrapper around the HTML 5 Drag API.
 */
@Directive({
    selector: '[anjDraggable]'
})
export class DraggableDirective {

    /**
     * @description Binds to arbitrary data to be passed via anjDroppable's onDrop event emitter.
     */
    @Input() public anjDraggable: any;
    @HostBinding('attr.draggable') public readonly isDraggable: boolean;
    @HostBinding('class.dragging') public isDragging: boolean;

    constructor() {
        this.isDraggable = true;
    }

    @HostListener('dragstart', ['$event'])
    public handleDragStart(event: DragEvent) {
        this.isDragging = true;

        if (event.dataTransfer) {
            event.dataTransfer.effectAllowed = 'move';
            event.dataTransfer.setData('data', JSON.stringify(this.anjDraggable));
        }
    }

    @HostListener('dragend', ['$event'])
    public handleDragEnd(event: DragEvent) {
        event.preventDefault();
        this.isDragging = false;
    }
}
