import {Directive, Output, EventEmitter, HostBinding, HostListener} from '@angular/core';

import {DropData} from '../interfaces/DropData';

/**
 * @description A wrapper around the HTML 5 Drop API.
 */
@Directive({
    selector: '[anjDroppable]'
})
export class DroppableDirective {
    /**
     * @description Emitted after and HTML drop event. Emits data bound to anjDraggable.
     */
    @Output() public anjDrop: EventEmitter<DropData>;
    @HostBinding('class.drag-over') public isDraggedOver: boolean;

    constructor() {
        this.anjDrop = new EventEmitter<DropData>();
    }

    @HostListener('dragenter', ['$event'])
    public handleDragEnter(): void {
        this.isDraggedOver = true;
    }

    @HostListener('dragleave', ['$event'])
    public handleDragLeave(): void {
        this.isDraggedOver = false;
    }

    @HostListener('dragover', ['$event'])
    public handleDragOver(event: DragEvent): boolean {
        if (event.preventDefault) {
            event.preventDefault();
        }

        event.dataTransfer.dropEffect = 'move';
        return false;
    }

    @HostListener('drop', ['$event'])
    public handleDrop(event: DragEvent): boolean {
        if (event.stopPropagation) {
            event.stopPropagation(); // Stops some browsers from redirecting.
        }

        this.isDraggedOver = false;
        if (event.dataTransfer && event.dataTransfer.getData && event.dataTransfer.getData('data')) {
            const data = JSON.parse(event.dataTransfer.getData('data'));
            this.anjDrop.emit({payload: data});
        }

        return false;
    }
}
