import {Attribute, Directive, Output, EventEmitter, OnInit, OnDestroy, ElementRef} from '@angular/core';

/** @internal */
@Directive({
    selector: '[anjResizer]'
})
export class ResizerDirective implements OnInit, OnDestroy {

    @Output() public resize: EventEmitter<null>;

    private parentElement: HTMLElement;
    private resizeListener: EventListener;
    private resizeDebouncer: number;
    private parentWidth: number;
    private parentHeight: number;

    constructor(@Attribute('anjResizer') public anjResizer: number,
                private element: ElementRef) {
        this.resize = new EventEmitter<null>();
        this.resizeListener = this.handleResize.bind(this);
    }

    public ngOnInit(): void {
        window.addEventListener('resize', this.resizeListener);

        this.parentElement = this.element.nativeElement.parentElement;
        this.getDimensions();
    }

    public ngOnDestroy(): void {
        window.clearTimeout(this.resizeDebouncer);
        window.removeEventListener('resize', this.resizeListener);
    }

    public handleResize(): void {
        window.clearTimeout(this.resizeDebouncer);

        this.resizeDebouncer = window.setTimeout(() => {
            const needsResize: boolean = this.getDimensions();

            if (needsResize) {
                this.resize.emit();
            }
        }, this.anjResizer);
    }

    public getDimensions(): boolean {
        const parentWidth = this.parentElement.clientWidth;
        const parentHeight = this.parentElement.clientHeight;

        if (parentWidth === this.parentWidth && parentHeight === this.parentHeight) {
            return false;
        }

        this.parentWidth = parentWidth;
        this.parentHeight = parentHeight;

        return true;
    }
}
