# Transition

The TransitionDirective uses css classes to trigger CSS animations which are well supported across 
browsers and is much, much lighter

We bundle the basic route transition in with the global css. It is called "routefade" that should be used in "in" mode. However 
you can name and define your own transitions as you please. To use this transition, you need to wrap your router-outlet in a block
level element (another component, a div, whatever)

```html
<div [anjTransition]="getRouterState(o)" name="routefade" mode="in">
    <router-outlet #o="outlet"></router-outlet>
</div>
```

Then you need to update the state via routing. RouteData works well for this. In your routes, you can add a property called state.
```typescript
{
    path: '',
    component: HomeComponent,
    data: {
        state: 'home'
    },
}
```

Lastly, you need to get that state to the transition. In the component that hosts the router outlet, add a function that returns
the current state.

```typescript
public getRouterState(outlet: RouterOutlet): RouterState {
    const routeData = outlet.activatedRouteData as any;
    return routeData.state;
}
```
