import {Directive, ElementRef, HostBinding, HostListener, Optional, Self} from '@angular/core';
import {ControlValueAccessor, FormGroupDirective, NgControl, NgForm} from '@angular/forms';

@Directive({
    /* tslint:disable:directive-selector */
    selector: 'anj-input'
})
export class InputValueDirective implements ControlValueAccessor {

    public onChange: (value: any) => void;
    public onTouched: () => void;

    @HostBinding('attr.invalid') get isInvalid(): boolean {
        if (!this.ngControl) {
            return false;
        }

        const isSubmitted = (this.parent && this.parent.submitted);
        return this.ngControl.invalid && (this.ngControl.touched || isSubmitted);
    }

    private readonly parent: NgForm | FormGroupDirective | null;

    constructor(
        @Optional() parentForm: NgForm,
        @Optional() parentFormGroup: FormGroupDirective,
        @Optional() @Self() private ngControl: NgControl,
        private element: ElementRef
    ) {
        this.onChange = () => {/**/};
        this.onTouched = () => {/**/};

        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }

        this.parent = parentForm ? parentForm : parentFormGroup ? parentFormGroup : null;
    }

    @HostListener('anjChange', ['$event.detail.value'])
    public _onChange(value: any) {
        this.onChange(value);
    }

    @HostListener('anjBlur')
    public _onBlur() {
        this.onTouched();
    }

    public writeValue(value: any) {
        this.element.nativeElement.value = value;
    }

    public registerOnChange(fn: (value: any) => void) {
        this.onChange = fn;
    }

    public registerOnTouched(fn: () => void) {
        this.onTouched = fn;
    }

    public setDisabledState(disabled: boolean) {
        this.element.nativeElement.disabled = disabled;
    }
}
