import {Directive, HostBinding, Input, OnChanges} from '@angular/core';
import {NG_VALIDATORS, Validator, ValidatorFn, FormControl, AbstractControl} from '@angular/forms';

export function validateMaxValueFactory(max: number = 0): ValidatorFn {
    return (c: AbstractControl) => {
        if (c.value === undefined || c.value === null) {
            return null;
        }

        if (Number(c.value) <= max) {
            return null;
        } else {
            return {
                max: {
                    valid: false
                }
            };
        }
    };
}

/* tslint:disable:directive-selector */
@Directive({
    selector: '[max][ngModel]',
    providers: [
        {provide: NG_VALIDATORS, useExisting: MaxValidatorDirective, multi: true}
    ]
})
export class MaxValidatorDirective implements OnChanges, Validator {

    @HostBinding('attr.max') public _max: number;

    @Input()
    get max(): number { return this._max || 0; }
    set max(max: number) {
        this._max = Number(max);
    }

    private validator: ValidatorFn;

    public ngOnChanges() {
        this.validator = validateMaxValueFactory(this.max);
    }

    public validate(c: FormControl) {
        return this.validator(c);
    }

}
