import {Directive, HostBinding, Input, OnChanges} from '@angular/core';
import {NG_VALIDATORS, Validator, ValidatorFn, FormControl, AbstractControl} from '@angular/forms';

export function validateMinValueFactory(min: number = 0): ValidatorFn {
    return (c: AbstractControl) => {
        if (c.value === undefined || c.value === null) {
            return null;
        }

        if (Number(c.value) >= min) {
            return null;
        } else {
            return {
                min: {
                    valid: false
                }
            };
        }
    };
}

/* tslint:disable:directive-selector */
@Directive({
    selector: '[min][ngModel]',
    providers: [
        {provide: NG_VALIDATORS, useExisting: MinValidatorDirective, multi: true}
    ]
})
export class MinValidatorDirective implements OnChanges, Validator {

    @HostBinding('attr.min') public _min: number;

    @Input()
    get min(): number { return this._min || 0; }
    set min(min: number) {
        this._min = Number(min);
    }

    private validator: ValidatorFn;

    public ngOnChanges() {
        this.validator = validateMinValueFactory(this.min);
    }

    public validate(c: FormControl) {
        return this.validator(c);
    }

}
