import {Directive, ElementRef, EventEmitter, HostBinding, HostListener, Optional, Output, Self} from '@angular/core';
import {ControlValueAccessor, FormGroupDirective, NgControl, NgForm} from '@angular/forms';

@Directive({
    /* tslint:disable-next-line:directive-selector */
    selector: 'anj-radio-group, anj-select, anj-type-ahead',
})
export class SelectValueDirective implements ControlValueAccessor {

    /**
     * Enable two-way binding
     */
    @Output() public readonly valueChange: EventEmitter<any> = new EventEmitter<any>();

    public onChange: (value: any) => void;
    public onTouched: () => void;

    @HostBinding('attr.invalid') get isInvalid(): boolean {
        if (!this.ngControl) {
            return false;
        }

        const isSubmitted = (this.parent && this.parent.submitted);
        return this.ngControl.invalid && (this.ngControl.touched || isSubmitted);
    }

    private readonly parent: NgForm | FormGroupDirective | null;

    constructor(
        @Optional() parentForm: NgForm,
        @Optional() parentFormGroup: FormGroupDirective,
        @Optional() @Self() private ngControl: NgControl,
        private element: ElementRef
    ) {
        this.onChange = () => {/**/};
        this.onTouched = () => {/**/};

        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }

        this.parent = parentForm ? parentForm : parentFormGroup ? parentFormGroup : null;
    }

    public writeValue(value: any) {
        this.element.nativeElement.value = value;
    }

    @HostListener('anjChange', ['$event.target'])
    public _onChange(target: HTMLInputElement) {
        // ignore the input for type-aheads
        if (target.matches('anj-input')) {
            return;
        }

        const value = target.value;
        this.onChange(value);
        this.valueChange.emit(value);
    }

    @HostListener('anjBlur')
    public _onBlur() {
        this.onTouched();
    }

    public registerOnChange(fn: (value: any) => void) {
        this.onChange = fn;
    }

    public registerOnTouched(fn: () => void) {
        this.onTouched = fn;
    }

    public setDisabledState(isDisabled: boolean) {
        this.element.nativeElement.disabled = isDisabled;
    }
}
