import {
    Directive,
    Optional,
    ElementRef,
    Inject,
    AfterViewInit,
    LOCALE_ID, Self, EventEmitter, Input, HostBinding, HostListener, OnChanges
} from '@angular/core';

import {TENANT_MERIDIEM, TENANT_TIMEZONE_STRING} from '../../common/interfaces/Tokens';
import {ControlValueAccessor, NgControl} from '@angular/forms';

@Directive({
    selector: 'anj-input[anjTimePicker]',
    exportAs: 'anjTimepickerInput'
})
export class TimePickerDirective implements AfterViewInit, ControlValueAccessor {
    public readonly valueChange: EventEmitter<string | null>;

    @HostBinding('value') public value: string;

    constructor(private _element: ElementRef,
                @Optional() @Self() private ngControl?: NgControl,
                @Optional() @Inject(LOCALE_ID) private _locale?: string,
                @Optional() @Inject(TENANT_TIMEZONE_STRING) private _timezone?: string,
                @Optional() @Inject(TENANT_MERIDIEM) private _meridiem?: string) {
        this.valueChange = new EventEmitter<string | null>();

        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }

    ngAfterViewInit() {
       const picker = this._element.nativeElement.children[0];

       picker.setAttribute('locale', this._locale);
       picker.setAttribute('meridiem', this._meridiem);
       picker.setAttribute('timezone', this._timezone);
    }

    // // Start: ControlValueAccessor requirements
    public writeValue(newValue: string): void {
        this.value = newValue;
    }

    public onChange: Function = () => {
    }

    public onTouch: Function = () => {
    }

    public registerOnChange(fn: any): void {
        this.onChange = fn;
    }

    public registerOnTouched(fn: any): void {
        this.onTouch = fn;
    }
    // // End: ControlValueAccessor requirements

    @HostListener('anjTimeChanged', ['$event'])
    public onInput(event: any) {

        this.value = event.detail;
        if (this.ngControl) {
            this.ngControl.control.setValue(this.value);
        }

        this.valueChange.emit(this.value);
    }
}
