import {DebugElement, CUSTOM_ELEMENTS_SCHEMA} from '@angular/core';
import {TestBed, ComponentFixture, async} from '@angular/core/testing';
import {By} from '@angular/platform-browser';
import {FormsModule, ReactiveFormsModule} from '@angular/forms';

import {BooleanValueDirective} from '@anjuna/angular-core';
import {StubBooleanValueComponent, StubBooleanValueNGMComponent, StubBooleanValueMDComponent} from '@anjuna/angular-core/testing/src/boolean-value.mock';

describe('Directive: boolean-value', () => {
    let component: StubBooleanValueComponent;
    let fixture: ComponentFixture<StubBooleanValueComponent>;
    let dir: any;

    beforeEach(async(() => {
        TestBed.configureTestingModule({
            declarations: [
                BooleanValueDirective,
                StubBooleanValueComponent
            ],
            schemas: [
                CUSTOM_ELEMENTS_SCHEMA
            ]
        }).compileComponents().then(() => {
            fixture = TestBed.createComponent(StubBooleanValueComponent);
            component = fixture.componentInstance;
            dir = component.directive as any;
            fixture.detectChanges();
        });
    }));

    describe('Public API', () => {
        it('should exist', () => {
            // these only get defined up front to not break things if there's not ng form
            component.directive.onChange(true);
            component.directive.onTouched();
            expect(component.directive).toBeTruthy();
        });

        it('should set the disabled state', () => {
            component.directive.setDisabledState(true);
            const zCheck: DebugElement = fixture.debugElement.query(By.css('anj-checkbox'));
            expect(zCheck.nativeElement.disabled).toBeTruthy();
        });

        it('should have applicable functions for ControlValueAccessor', () => {
            component.directive.onChange(null);
            component.directive.onTouched();
        });
    });
});

describe('Directive: boolean-value - template driven', () => {
    let component: StubBooleanValueNGMComponent;
    let fixture: ComponentFixture<StubBooleanValueNGMComponent>;

    beforeEach(async(() => {
        TestBed.configureTestingModule({
            imports: [
                FormsModule,
            ],
            declarations: [
                BooleanValueDirective,
                StubBooleanValueNGMComponent
            ],
            schemas: [
                CUSTOM_ELEMENTS_SCHEMA
            ]
        }).compileComponents().then(() => {
            fixture = TestBed.createComponent(StubBooleanValueNGMComponent);
            component = fixture.componentInstance;
            fixture.detectChanges();
        });
    }));

    describe('Public API', () => {
        it('should exist', () => {
            expect(component.directive).toBeTruthy();
        });

        it('should mark the control as touched when onBlurChange is called', () => {
            spyOn(component.directive, 'onTouched');
            component.directive._onBlur();

            expect(component.directive.onTouched).toHaveBeenCalled();
        });

        it('should call on onChange when onCheckedChange is called', () => {
            spyOn(component.directive, 'onChange');
            component.directive._onChange(true);

            expect(component.directive.onChange).toHaveBeenCalledWith(true);
        });
    });
});

describe('Directive: boolean-value - model driven', () => {
    let component: StubBooleanValueMDComponent;
    let fixture: ComponentFixture<StubBooleanValueMDComponent>;
    let dir: any;

    beforeEach(async(() => {
        TestBed.configureTestingModule({
            imports: [
                ReactiveFormsModule,
            ],
            declarations: [
                BooleanValueDirective,
                StubBooleanValueMDComponent
            ],
            schemas: [
                CUSTOM_ELEMENTS_SCHEMA
            ]
        }).compileComponents().then(() => {
            fixture = TestBed.createComponent(StubBooleanValueMDComponent);
            component = fixture.componentInstance;
            dir = component.directive as any;
            fixture.detectChanges();
        });
    }));

    describe('Public API', () => {
        it('should exist', () => {
            expect(component.directive).toBeTruthy();
        });
    });
});
