import {DebugElement} from '@angular/core';
import {TestBed, ComponentFixture, async} from '@angular/core/testing';
import {By} from '@angular/platform-browser';

import {DraggableDirective} from '@anjuna/angular-core';
import {StubDragComponent} from '../testing/src/stub-drag.mock';

const testData = {foo: 'bar'};

describe('directive: anjDraggable', () => {
    let directive: DraggableDirective;

    beforeEach(() => {
        directive = new DraggableDirective();
        directive.anjDraggable = testData;
    });

    it('should exist', () => {
        expect(typeof directive).not.toBe('undefined');
    });

    it('should bind to the input', () => {
        expect(directive.anjDraggable.foo).toEqual(testData.foo);
    });
});

describe('directive: anjDraggable integration', () => {
    let component: StubDragComponent;
    let fixture: ComponentFixture<StubDragComponent>;

    beforeEach(async(() => {
        TestBed.configureTestingModule({
            declarations: [
                StubDragComponent,
                DraggableDirective
            ]
        });
    }));

    beforeEach(() => {
        fixture = TestBed.createComponent(StubDragComponent);
        component = fixture.componentInstance;

        fixture.detectChanges();
    });

    it('should reflect the draggable state in the UI', () => {
        const draggable: DebugElement = fixture.debugElement.query(By.css('.comp-drag'));

        fixture.detectChanges();

        expect(draggable.nativeElement.draggable).toBeTruthy();
    });

    it('should reflect the dragging state in the UI', () => {
        const draggable: DebugElement = fixture.debugElement.query(By.css('.comp-drag'));
        // Hack a drag event
        const event: Event = new Event('dragstart');

        draggable.nativeElement.dispatchEvent(event);
        fixture.detectChanges();

        expect(draggable.nativeElement.classList).toContain('dragging');
    });

    it('should bind the data to the event', () => {
        const draggable: DebugElement = fixture.debugElement.query(By.css('.comp-drag'));
        // Hack a drag event
        const event: any = new Event('dragstart');

        event.dataTransfer = {
            effectAllowed: null,
            data: null,
            setData: (f: string, data: any) => {
                event.data = data;
            }
        };

        draggable.nativeElement.dispatchEvent(event);
        fixture.detectChanges();

        expect(event.data).toEqual(component.data.toString());
    });

    it('should reflect the no longer dragging state in the UI', () => {
        const draggable: DebugElement = fixture.debugElement.query(By.css('.comp-drag'));
        // Hack a drag event
        const startEvent: Event = new Event('dragstart');
        const endEvent: Event = new Event('dragend');

        draggable.nativeElement.dispatchEvent(startEvent);
        fixture.detectChanges();

        draggable.nativeElement.dispatchEvent(endEvent);
        fixture.detectChanges();

        expect(draggable.nativeElement.classList).not.toContain('dragging');
    });
});
