import {DebugElement} from '@angular/core';
import {TestBed, ComponentFixture, async} from '@angular/core/testing';
import {By} from '@angular/platform-browser';

import {DroppableDirective} from '@anjuna/angular-core';
import {StubDropComponent} from '../testing/src/stub-drop.mock';

const testData = {foo: 'bar'};

describe('directive: anjDroppable', () => {
    let directive: DroppableDirective;

    beforeEach(() => {
        directive = new DroppableDirective();
    });

    it('should exist', () => {
        expect(typeof directive).not.toBe('undefined');
    });

    it('should have an output for the onDrop event', () => {
        expect(typeof directive.anjDrop).not.toBe('undefined');
        expect(typeof directive.anjDrop).toBe('object');
    });

    it('should handleDragOver', () => {
        const event = {
            dataTransfer: {}
        };

        expect(directive.handleDragOver(event as DragEvent)).toBeFalsy();
    });

    it('should handleDrop', () => {
        const event = {
            dataTransfer: {}
        };

        expect(directive.handleDrop(event as DragEvent)).toBeFalsy();
    });
});

describe('directive: anjDroppable integration', () => {
    let component: StubDropComponent;
    let fixture: ComponentFixture<StubDropComponent>;

    beforeEach(async(() => {
        TestBed.configureTestingModule({
            declarations: [
                StubDropComponent,
                DroppableDirective
            ]
        });
    }));

    beforeEach(() => {
        fixture = TestBed.createComponent(StubDropComponent);
        component = fixture.componentInstance;

        fixture.detectChanges();
    });

    it('should reflect the drag enter state in the UI', () => {
        const droppable: DebugElement = fixture.debugElement.query(By.css('.comp-drop'));
        const event: Event = new Event('dragenter');

        droppable.nativeElement.dispatchEvent(event);
        fixture.detectChanges();

        expect(droppable.nativeElement.classList).toContain('drag-over');
    });

    it('should reflect the drag leave state in the UI', () => {
        const droppable: DebugElement = fixture.debugElement.query(By.css('.comp-drop'));
        const enterEvent: Event = new Event('dragenter');
        const leaveEvent: Event = new Event('dragleave');

        droppable.nativeElement.dispatchEvent(enterEvent);
        fixture.detectChanges();

        droppable.nativeElement.dispatchEvent(leaveEvent);
        fixture.detectChanges();

        expect(droppable.nativeElement.classList).not.toContain('drag-over');
    });

    it('should update the drag event on dragover', () => {
        const droppable: DebugElement = fixture.debugElement.query(By.css('.comp-drop'));
        const event: any = new Event('dragover');

        event.dataTransfer = {
            dropEffect: null
        };

        droppable.nativeElement.dispatchEvent(event);
        fixture.detectChanges();

        expect(event.dataTransfer.dropEffect).toEqual('move');
    });

    it('should reflected the not drag over state on drop', () => {
        const droppable: DebugElement = fixture.debugElement.query(By.css('.comp-drop'));
        const enterEvent: Event = new Event('dragenter');
        const dropEvent: Event = new Event('drop');

        droppable.nativeElement.dispatchEvent(enterEvent);
        fixture.detectChanges();

        droppable.nativeElement.dispatchEvent(dropEvent);
        fixture.detectChanges();

        expect(droppable.nativeElement.classList).not.toContain('drag-over');
    });

    it('should emit the onDrop event on drop', () => {
        spyOn(component, 'handleDrop');

        const droppable: DebugElement = fixture.debugElement.query(By.css('.comp-drop'));
        const event: any = new Event('drop');

        event.dataTransfer = {
            getData: (format: string) => {
                if (format === 'data') {
                    return JSON.stringify(testData);
                }
            }
        };

        droppable.nativeElement.dispatchEvent(event);
        fixture.detectChanges();

        expect(component.handleDrop).toHaveBeenCalledWith({payload: testData});
    });
});
