import {DebugElement, CUSTOM_ELEMENTS_SCHEMA} from '@angular/core';
import {TestBed, ComponentFixture, async} from '@angular/core/testing';
import {By} from '@angular/platform-browser';
import {FormsModule, ReactiveFormsModule} from '@angular/forms';

import {InputValueDirective} from '@anjuna/angular-core';
import {StubInputValueComponent, StubInputValueNGMComponent, StubInputValueMDComponent} from '@anjuna/angular-core/testing/src/input-value.mock';

describe('Directive: input-value', () => {
    let component: StubInputValueComponent;
    let fixture: ComponentFixture<StubInputValueComponent>;
    let dir: any;

    beforeEach(async(() => {
        TestBed.configureTestingModule({
            declarations: [
                InputValueDirective,
                StubInputValueComponent
            ],
            schemas: [
                CUSTOM_ELEMENTS_SCHEMA
            ]
        }).compileComponents().then(() => {
            fixture = TestBed.createComponent(StubInputValueComponent);
            component = fixture.componentInstance;
            dir = component.directive as any;
            fixture.detectChanges();
        });
    }));

    describe('Public API', () => {
        it('should exist', () => {
            // these only get defined up front to not break things if there's not ng form
            component.directive.onChange('Sushi');
            component.directive.onTouched();
            expect(component.directive).toBeTruthy();
        });

        it('should set the disabled state', () => {
            component.directive.setDisabledState(true);
            const zCheck: DebugElement = fixture.debugElement.query(By.css('anj-input'));
            expect(zCheck.nativeElement.disabled).toBeTruthy();
        });

        it('should have applicable functions for ControlValueAccessor', () => {
            component.directive.onChange(null);
            component.directive.onTouched();
        });
    });
});

describe('Directive: input-value - template driven', () => {
    let component: StubInputValueNGMComponent;
    let fixture: ComponentFixture<StubInputValueNGMComponent>;

    beforeEach(async(() => {
        TestBed.configureTestingModule({
            imports: [
                FormsModule,
            ],
            declarations: [
                InputValueDirective,
                StubInputValueNGMComponent
            ],
            schemas: [
                CUSTOM_ELEMENTS_SCHEMA
            ]
        }).compileComponents().then(() => {
            fixture = TestBed.createComponent(StubInputValueNGMComponent);
            component = fixture.componentInstance;
            fixture.detectChanges();
        });
    }));

    describe('Public API', () => {
        it('should exist', () => {
            expect(component.directive).toBeTruthy();
        });

        it('should mark the control as touched when onBlurChange is called', () => {
            spyOn(component.directive, 'onTouched');
            component.directive._onBlur();

            expect(component.directive.onTouched).toHaveBeenCalled();
        });

        it('should call on onChange when onCheckedChange is called', () => {
            spyOn(component.directive, 'onChange');
            component.directive._onChange(true);

            expect(component.directive.onChange).toHaveBeenCalledWith(true);
        });
    });
});

describe('Directive: input-value - model driven', () => {
    let component: StubInputValueMDComponent;
    let fixture: ComponentFixture<StubInputValueMDComponent>;
    let dir: any;

    beforeEach(async(() => {
        TestBed.configureTestingModule({
            imports: [
                ReactiveFormsModule,
            ],
            declarations: [
                InputValueDirective,
                StubInputValueMDComponent
            ],
            schemas: [
                CUSTOM_ELEMENTS_SCHEMA
            ]
        }).compileComponents().then(() => {
            fixture = TestBed.createComponent(StubInputValueMDComponent);
            component = fixture.componentInstance;
            dir = component.directive as any;
            fixture.detectChanges();
        });
    }));

    describe('Public API', () => {
        it('should exist', () => {
            expect(component.directive).toBeTruthy();
        });
    });
});
