import {MaxValidatorDirective, validateMaxValueFactory} from '@anjuna/angular-core';

const mockControl: any = {
    value: null
};

describe('directive: validator max', () => {
    let directive: MaxValidatorDirective;

    beforeEach(() => {
        directive = new MaxValidatorDirective();
    });

    it('should exist', () => {
        expect(typeof directive).not.toBe('undefined');
    });

    it('should have an input for the maximum value', () => {
        expect(typeof directive.max).not.toBe('undefined');
    });

    it('should take a normal form param (string) and parse it', () => {
        const dir = directive as any;
        dir.max = '10';

        expect(directive.max).toEqual(10);
    });

    it('should default to 0', () => {
        expect(directive.max).toEqual(0);
    });

    it('should return null when the control has a value over = max', () => {
        directive.ngOnChanges();

        mockControl.value = 0;
        const error = directive.validate(mockControl);

        expect(error).toEqual(null);
    });

    it('should return null when the control has a value over < max', () => {
        directive.ngOnChanges();

        mockControl.value = -5;
        const error = directive.validate(mockControl);

        expect(error).toEqual(null);
    });

    it('should return null when the control has a null value', () => {
        directive.ngOnChanges();

        mockControl.value = null;
        const error = directive.validate(mockControl);

        expect(error).toEqual(null);
    });

    it('should return an error object when the control has a value over > max', () => {
        directive.ngOnChanges();

        mockControl.value = 5;
        const error = directive.validate(mockControl);

        expect(error).toEqual({
            max: {
                valid: false
            }
        });
    });

    it('should re-evaluate the validator function when the min changes', () => {
        directive.max = 10;
        directive.ngOnChanges();
        mockControl.value = 5;

        const error1 = directive.validate(mockControl);

        expect(error1).toEqual(null);

        directive.max = 1;
        directive.ngOnChanges();

        const error2 = directive.validate(mockControl);

        expect(error2).not.toEqual(null);
    });

    it('should take have a default arg to the validator factory', () => {
        expect(validateMaxValueFactory()).toBeTruthy();
    });
});
