import {MinValidatorDirective, validateMinValueFactory} from '@anjuna/angular-core';

const mockControl: any = {
    value: null
};

describe('directive: validator min', () => {
    let directive: MinValidatorDirective;

    beforeEach(() => {
        directive = new MinValidatorDirective();
    });

    it('should exist', () => {
        expect(typeof directive).not.toBe('undefined');
    });

    it('should have an input for the minimum value', () => {
        expect(typeof directive.min).not.toBe('undefined');
    });

    it('should take a normal form param (string) and parse it', () => {
        const dir = directive as any;
        dir.min = '10';

        expect(directive.min).toEqual(10);
    });

    it('should default to 0', () => {
        expect(directive.min).toEqual(0);
    });

    it('should return null when the control has a value over = min', () => {
        directive.ngOnChanges();

        mockControl.value = 0;
        const error = directive.validate(mockControl);

        expect(error).toEqual(null);
    });

    it('should return null when the control has a value over > min', () => {
        directive.ngOnChanges();

        mockControl.value = 5;
        const error = directive.validate(mockControl);

        expect(error).toEqual(null);
    });

    it('should return null when the control has a null value', () => {
        directive.ngOnChanges();

        mockControl.value = null;
        const error = directive.validate(mockControl);

        expect(error).toEqual(null);
    });

    it('should return an error object when the control has a value over < min', () => {
        directive.ngOnChanges();

        mockControl.value = -5;
        const error = directive.validate(mockControl);

        expect(error).toEqual({
            min: {
                valid: false
            }
        });
    });

    it('should re-evaluate the validator function when the min changes', () => {
        directive.ngOnChanges();
        mockControl.value = 0;

        const error1 = directive.validate(mockControl);

        expect(error1).toEqual(null);

        directive.min = 10;
        directive.ngOnChanges();

        const error2 = directive.validate(mockControl);

        expect(error2).not.toEqual(null);
    });

    it('should take have a default arg to the validator factory', () => {
        expect(validateMinValueFactory()).toBeTruthy();
    });
});
