import {ElementRef} from '@angular/core';
import {TestBed, ComponentFixture, async, fakeAsync} from '@angular/core/testing';

import {ResizerDirective} from '@anjuna/angular-core';
import {StubResizerComponent} from '../testing/src/stub-resizer.mock';
import {advance} from '../testing/src/helpers';

describe('directive: anjResizer', () => {
    let directive: ResizerDirective;

    beforeEach(() => {
        directive = new ResizerDirective(50, new ElementRef('<div></div>'));
    });

    it('should exist', () => {
        expect(typeof directive).not.toBe('undefined');
    });

    it('should have an input for the debounceTime', () => {
        expect(typeof directive.anjResizer).not.toBe('undefined');
        expect(typeof directive.anjResizer).toBe('number');
        expect(directive.anjResizer).toBe(50);
    });

    it('should have an output for the onResize event', () => {
        expect(typeof directive.resize).not.toBe('undefined');
        expect(typeof directive.resize).toBe('object');
    });

    it('should do nothing if the dimension have no changed', () => {
        const dir = directive as any;

        dir.parentWidth = 200;
        dir.parentHeight = 200;

        dir.parentElement = {
            clientWidth: 200,
            clientHeight: 200
        };

        expect(directive.getDimensions()).toBeFalsy();
    });
});

describe('directive: anjResizer integration', () => {
    let component: StubResizerComponent;
    let fixture: ComponentFixture<StubResizerComponent>;

    beforeEach(async(() => {
        TestBed.configureTestingModule({
            declarations: [
                StubResizerComponent,
                ResizerDirective
            ]
        });
    }));

    beforeEach(() => {
        fixture = TestBed.createComponent(StubResizerComponent);
        component = fixture.componentInstance;

        fixture.detectChanges();
    });

    it('should bubble up a resize event when calculated as such', fakeAsync(() => {
        spyOn(component, 'handleResize');
        spyOn(component.resizer, 'getDimensions').and.callFake(() => true);

        const event: Event = new Event('resize');
        window.dispatchEvent(event);

        advance(fixture, 50);

        expect(component.handleResize).toHaveBeenCalled();
    }));

    it('should not bubble up a resize event when calculated as such', fakeAsync(() => {
        spyOn(component, 'handleResize');
        spyOn(component.resizer, 'getDimensions').and.callFake(() => false);

        const event: Event = new Event('resize');
        window.dispatchEvent(event);

        advance(fixture, 50);

        expect(component.handleResize).not.toHaveBeenCalled();
    }));
});
