import {DebugElement, CUSTOM_ELEMENTS_SCHEMA} from '@angular/core';
import {TestBed, ComponentFixture, async} from '@angular/core/testing';
import {By} from '@angular/platform-browser';
import {FormsModule, ReactiveFormsModule} from '@angular/forms';

import {SelectValueDirective} from '@anjuna/angular-core';
import {StubSelectValueComponent, StubSelectValueNGMComponent, StubSelectValueMDComponent} from '@anjuna/angular-core/testing/src/select-value.mock';

describe('Directive: select-value', () => {
    let component: StubSelectValueComponent;
    let fixture: ComponentFixture<StubSelectValueComponent>;
    let dir: any;

    beforeEach(async(() => {
        TestBed.configureTestingModule({
            declarations: [
                SelectValueDirective,
                StubSelectValueComponent
            ],
            schemas: [
                CUSTOM_ELEMENTS_SCHEMA
            ]
        }).compileComponents().then(() => {
            fixture = TestBed.createComponent(StubSelectValueComponent);
            component = fixture.componentInstance;
            dir = component.directive as any;
            fixture.detectChanges();
        });
    }));

    describe('Public API', () => {
        it('should exist', () => {
            // these only get defined up front to not break things if there's not ng form
            component.directive.onChange('Sushi');
            component.directive.onTouched();
            expect(component.directive).toBeTruthy();
        });

        it('should set the disabled state', () => {
            component.directive.setDisabledState(true);
            const zCheck: DebugElement = fixture.debugElement.query(By.css('anj-radio-group'));
            expect(zCheck.nativeElement.disabled).toBeTruthy();
        });

        it('should have applicable functions for ControlValueAccessor', () => {
            component.directive.onChange(null);
            component.directive.onTouched();
        });
    });
});

describe('Directive: select-value - template driven', () => {
    let component: StubSelectValueNGMComponent;
    let fixture: ComponentFixture<StubSelectValueNGMComponent>;

    beforeEach(async(() => {
        TestBed.configureTestingModule({
            imports: [
                FormsModule,
            ],
            declarations: [
                SelectValueDirective,
                StubSelectValueNGMComponent
            ],
            schemas: [
                CUSTOM_ELEMENTS_SCHEMA
            ]
        }).compileComponents().then(() => {
            fixture = TestBed.createComponent(StubSelectValueNGMComponent);
            component = fixture.componentInstance;
            fixture.detectChanges();
        });
    }));

    describe('Public API', () => {
        it('should exist', () => {
            expect(component.directive).toBeTruthy();
        });

        it('should mark the control as touched when onBlurChange is called', () => {
            spyOn(component.directive, 'onTouched');
            component.directive._onBlur();

            expect(component.directive.onTouched).toHaveBeenCalled();
        });

        it('should call on onChange when _onChange is called', () => {
            spyOn(component.directive, 'onChange');
            const radio = document.createElement('input');
            radio.value = 'Sushi';
            component.directive._onChange(radio);

            expect(component.directive.onChange).toHaveBeenCalledWith('Sushi');
        });

        it('should ignore change events from anj input', () => {
            spyOn(component.directive, 'onChange');

            const el: any = document.createElement('anj-input');
            component.directive._onChange(el);
            expect(component.directive.onChange).not.toHaveBeenCalled();
        });
    });
});

describe('Directive: select-value - model driven', () => {
    let component: StubSelectValueMDComponent;
    let fixture: ComponentFixture<StubSelectValueMDComponent>;
    let dir: any;

    beforeEach(async(() => {
        TestBed.configureTestingModule({
            imports: [
                ReactiveFormsModule,
            ],
            declarations: [
                SelectValueDirective,
                StubSelectValueMDComponent
            ],
            schemas: [
                CUSTOM_ELEMENTS_SCHEMA
            ]
        }).compileComponents().then(() => {
            fixture = TestBed.createComponent(StubSelectValueMDComponent);
            component = fixture.componentInstance;
            dir = component.directive as any;
            fixture.detectChanges();
        });
    }));

    describe('Public API', () => {
        it('should exist', () => {
            expect(component.directive).toBeTruthy();
        });
    });
});
