import {CUSTOM_ELEMENTS_SCHEMA, LOCALE_ID} from '@angular/core';
import {TestBed, ComponentFixture, async} from '@angular/core/testing';
import {FormsModule} from '@angular/forms';

import {TENANT_MERIDIEM, TENANT_TIMEZONE_STRING, TimePickerDirective} from '@anjuna/angular-core';
import {StubTimePickerComponent, StubTimePickerNoControlComponent} from '@anjuna/angular-core/testing/src/time-picker.mock';

describe('Directive: time-picker', () => {
    let component: StubTimePickerComponent;
    let fixture: ComponentFixture<StubTimePickerComponent>;

    beforeEach(async(() => {
        TestBed.configureTestingModule({
            imports: [
                FormsModule,
            ],
            declarations: [
                TimePickerDirective,
                StubTimePickerComponent
            ],
            schemas: [
                CUSTOM_ELEMENTS_SCHEMA
            ],
            providers: [
                {
                    provide: LOCALE_ID,
                    useValue: 'fr'
                },
                {
                    provide: TENANT_MERIDIEM,
                    useValue: false
                },
                {
                    provide: TENANT_TIMEZONE_STRING,
                    useValue: 'GST'
                }
            ]
        }).compileComponents().then(() => {
            fixture = TestBed.createComponent(StubTimePickerComponent);
            component = fixture.componentInstance;
            fixture.detectChanges();
        });
    }));

    describe('Public API', () => {
        it('should exist', () => {
            expect(component.directive).toBeTruthy();
        });

        it('should update the element after view init', () => {
            component.directive.ngAfterViewInit();

            const element = component.directive['_element'].nativeElement.children[0];

            expect(element.getAttribute('locale')).toEqual('fr');
            expect(element.getAttribute('meridiem')).toEqual('false');
            expect(element.getAttribute('timezone')).toEqual('GST');
        });

        it('should handle a value update from the picker', () => {
            spyOn(component.directive.valueChange, 'emit');
            component.directive.onInput({detail: '1:11'});

            expect(component.directive.value).toEqual('1:11');
            expect(component.directive['ngControl'].control.value).toEqual('1:11');
            expect(component.directive.valueChange.emit).toHaveBeenCalled();
        });
    });
});


describe('Directive: time-picker (no ngControl)', () => {
    let component: StubTimePickerNoControlComponent;
    let fixture: ComponentFixture<StubTimePickerNoControlComponent>;

    beforeEach(async(() => {
        TestBed.configureTestingModule({
            imports: [
                FormsModule,
            ],
            declarations: [
                TimePickerDirective,
                StubTimePickerNoControlComponent
            ],
            schemas: [
                CUSTOM_ELEMENTS_SCHEMA
            ],
            providers: [
                {
                    provide: LOCALE_ID,
                    useValue: 'fr'
                },
                {
                    provide: TENANT_MERIDIEM,
                    useValue: false
                },
                {
                    provide: TENANT_TIMEZONE_STRING,
                    useValue: 'GST'
                }
            ]
        }).compileComponents().then(() => {
            fixture = TestBed.createComponent(StubTimePickerNoControlComponent);
            component = fixture.componentInstance;
            fixture.detectChanges();
        });
    }));

    describe('Public API', () => {
        it('should exist', () => {
            expect(component.directive).toBeTruthy();
        });

        it('should handle a value update from the picker', () => {
            spyOn(component.directive.valueChange, 'emit');
            component.directive.onInput({detail: '1:11'});

            expect(component.directive.value).toEqual('1:11');
            expect(component.directive.valueChange.emit).toHaveBeenCalled();
        });

        it('should have applicable functions for ControlValueAccessor', () => {
            component.directive.onChange();
            component.directive.onTouch();
        });
    });
});
