import {ChangeDetectionStrategy, Component, ViewChild} from '@angular/core';
import {FormBuilder, FormControl, FormGroup, Validators} from '@angular/forms';

import {SelectValueDirective} from '@anjuna/angular-core';

@Component({
    template: `
        <form>
            <anj-radio-group name="favorite-food" [value]="'Burritos'">
                <anj-radio *ngFor="let food of foods" [value]="food">{{food}}</anj-radio>
            </anj-radio-group>
            <anj-button type="submit">Submit</anj-button>
        </form> `,
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class StubSelectValueComponent {
    @ViewChild(SelectValueDirective) public directive: SelectValueDirective;
    public foods = ['Sushi', 'Burritos', 'Sushi Burritos'];
}

@Component({
    template: `
        <form>
            <anj-radio-group name="favoriteFood" [(ngModel)]="favFood" required>
                <anj-radio *ngFor="let food of foods" [value]="food">{{food}}</anj-radio>
            </anj-radio-group>
            <anj-button type="submit">Submit</anj-button>
        </form>`
})
export class StubSelectValueNGMComponent {
    @ViewChild(SelectValueDirective) public directive: SelectValueDirective;

    public foods = ['Sushi', 'Burritos', 'Sushi Burritos'];
    public favFood = null;
}

@Component({
    template: `
        <form [formGroup]="foodForm">
            <anj-radio-group formControlName="favFood">
                <anj-radio *ngFor="let food of foods" [value]="food">{{food}}</anj-radio>
            </anj-radio-group>
            <anj-button-group>
                <anj-button type="submit">Submit</anj-button>
            </anj-button-group>
        </form>
    `
})
export class StubSelectValueMDComponent {
    @ViewChild(SelectValueDirective) public directive: SelectValueDirective;

    public foodForm: FormGroup;
    public favFood: FormControl;

    public foods = ['Sushi', 'Burritos', 'Sushi Burritos'];

    constructor(private builder: FormBuilder) {
        this.favFood = new FormControl(null, Validators.required);
        this.foodForm = builder.group({
            favFood: this.favFood
        });
    }
}
